type DiffResult = {
    t: 0 | 1 | 2;
    p: Array<string | number>;
    v?: unknown;
};

/**
 * Performs a deep difference between two objects.
 *
 * @example
 * diff({a: 1}, {a: 5}) //=> [{t: 2, p: ['a'], v: 5}]
 */
declare function diff(obj1: object, obj2: object): Array<DiffResult>;

/**
 * Performs a deep difference between two objects with custom comparator function.
 *
 * @example
 * diff({a: 1}, {a: 5}) //=> [{t: 2, p: ['a'], v: 5}]
 */
declare function diffWith(obj1: object, obj2: object, fn: (a: object, b: object) => boolean | undefined): Array<DiffResult>;

/**
 * You can apply the diff result onto the original object to get the modified object.
 *
 * @example
 * const a = {a: 1, b: 2};
 * const b = {a: 2, c: 3};
 * const out = patch(a, diff(a, b));
 * assert.deepStrictEqual(out, b); // ok
 */
declare function patch<T>(obj: T, patches: Array<DiffResult>): T;

export { type DiffResult, diff, diffWith, patch };
